use std::net::SocketAddr;

use fleck::fleck_core;

fn main() {
    pretty_env_logger::init_timed();

    let fleck = fleck::Fleck::new();
    let api = fleck.api();

    // generate ephemeral keypair
    api.with_service(|ns: &fleck_core::NodeService| {
        ns.self_node().gen_keypair();
    });

    // create UDP socket for communication with a local multicast group
    let sockref = api.with_service(|io: &fleck_core::io::IOService| {
        fleck_core::io::UdpSocketBuilder::default()
            .bind_to(SocketAddr::new(
                std::net::Ipv4Addr::UNSPECIFIED.into(),
                3535,
            ))
            .join_multicast("239.0.239.0".parse().unwrap(), "0.0.0.0".parse().unwrap())
            .build(io)
    });

    // enable local discovery
    api.with_service(|ld: &fleck_core::discovery::LocalDiscovery| {
        for peer in sockref.multicast_peers() {
            ld.add_multicast_peer(peer);
        }
    });

    api.run();
}
